import 'dart:async';

import 'package:flutter/material.dart';
import 'package:sunny_lifecycle/sunny_lifecycle.dart';
import 'package:sunny_sdk_core/model.dart';

IHelpService get helpService => sunny.get();

abstract class IHelpService implements LifecycleAware {
  Stream<String> get completedHelp;
  Future<Map<String, bool>> get done;
  Future reload({bool forceReload = false});

  Future checkHelp(BuildContext context, List<HelpSnippet> snippets);

  FutureOr<bool> isComplete(String key);

  Future markComplete(String key);

  UserPrefKey helpCompleteKey(String key);

  static const helpCompletePrefix = "helpComplete";
  FutureOr dispose();
}

// class HelpService with LoggingMixin implements LifecycleAware {
//   final UserPreferencesService userPrefs;
//   Map<String, bool> _currStatus;
//   final SafeCompleter<Map<String, bool>> _loaded = SafeCompleter();
//
//   Future<Map<String, bool>> get done => _loaded.future;
//   final StreamController<String> _completions = StreamController.broadcast();
//
//   Stream<String> get completedHelp => _completions.stream;
//
//   final _coachMark = CoachMark();
//
//   HelpService(this.userPrefs) {
//     userPrefs.registerPrefix(helpCompletePrefix);
//     reload();
//   }
//
//   Future reload({bool forceReload = false}) async {
//     if (_currStatus == null || forceReload) {
//       final allStatus = (await userPrefs.findByPrefix<bool>(helpCompletePrefix))
//           .map((key, value) {
//         return MapEntry(key.name.replaceFirst(helpCompletePrefix, ''), value);
//       });
//       _currStatus = allStatus;
//       _loaded.complete(_currStatus);
//     }
//   }
//
//   Future checkHelp(BuildContext context, List<HelpSnippet> snippets) async {
//     if (snippets.isNullOrEmpty) {
//       return;
//     }
//     await done;
//     final firstNotDone =
//         snippets.firstOrNull((snippet) => !_isComplete(snippet.helpKey));
//     if (firstNotDone == null) {
//       log.info("All help items are complete!");
//       return;
//     }
//
//     final screen = MediaQuery.of(context);
//     final screenWidth = screen.size.width;
//
//     log.info("Checking help for ${firstNotDone.helpKey}");
//     var target =
//         firstNotDone.widgetKey.currentContext?.findRenderObject() as RenderBox;
//     if (target == null) {
//       log.info("  not handling because the render target is missing");
//       return;
//     }
//     var globalCoords = target.localToGlobal(Offset.zero);
//     var targetSize = globalCoords & target.size;
//     var outline = Rect.fromCircle(
//       center: targetSize.center,
//       radius: targetSize.longestSide * 0.6,
//     );
//
//     var h = firstNotDone.height ?? 24.0;
//     _coachMark.show(
//         targetContext: firstNotDone.widgetKey.currentContext,
//         markRect: outline,
//         children: [
//           Positioned(
//             top: outline.top - (h + (8 * 2)),
//             left: 0,
//             width: screenWidth,
//             child: firstNotDone.help,
//           )
//         ],
//         duration: null,
//         onClose: () async {
//           await markComplete(firstNotDone.helpKey);
//           Future.delayed(300.ms, () {
//             log.info("Checking next help item");
//             return checkHelp(context, snippets);
//           });
//         });
//   }
//
//   FutureOr<bool> isComplete(String key) {
//     if (_currStatus != null) {
//       return SynchronousFuture(_currStatus[key] ?? false);
//     } else {
//       return done.then((value) => _currStatus[key] ?? false);
//     }
//   }
//
//   bool _isComplete(String key) {
//     assert(_currStatus != null);
//     return _currStatus[key] ?? false;
//   }
//
//   Future markComplete(String key) async {
//     _currStatus[key] = true;
//     _completions.add(key);
//     // return userPrefs.setTrue(helpCompleteKey(key));
//   }
//
//   UserPrefKey helpCompleteKey(String key) {
//     return UserPrefKey("$helpCompletePrefix$key");
//   }
//
//   static const helpCompletePrefix = "helpComplete";
//
//   @override
//   FutureOr dispose() {
//     _loaded.complete({});
//     return _completions.close();
//   }
// }

class HelpSnippet {
  final GlobalKey widgetKey;
  final String helpKey;
  final Widget help;
  final double height;

  const HelpSnippet({
    required this.widgetKey,
    required this.helpKey,
    required this.help,
    required this.height,
  });

  HelpSnippet.text(
      {required this.widgetKey,
      required this.helpKey,
      required String helpText,
      double fontSize = 24.0,
      this.height = 24.0,
      FontStyle fontStyle = FontStyle.italic,
      Color color = Colors.white})
      : help = Text(
          helpText,
          textAlign: TextAlign.center,
          style: TextStyle(
            fontSize: fontSize,
            fontStyle: fontStyle,
            color: color,
          ),
        );
}
