import 'dart:async';

import 'package:firebase_messaging/firebase_messaging.dart';
import 'package:flutter/src/widgets/framework.dart';
import 'package:sunny_dart/helpers.dart';
import 'package:sunny_lifecycle/push/push_interface.dart';
import 'package:sunny_lifecycle/push_notification_service.dart';

PushPlatform pushPlatform() => PushMessagingWeb._();

abstract class FirebaseMessagingWrapper {
  usePublicVapidKey(String string);
  Future requestPermission();
  Stream get onTokenRefresh;
  Future<String?> getToken();
  Stream<Map<String, dynamic>> get onMessage;
}

class PushMessagingWeb with PushPlatformMixin, Disposable, LoggingMixin implements PushPlatform {
  static bool _init = false;
  PushMessagingWeb._() : _mc = null {
    if (_mc == null) {
      log.warning("Firebase Messaging Disabled");
    }
    if (!_init) {
      _init = true;
      _mc?.usePublicVapidKey(
          'BNaq_veI9YJDhwllp7xbmiSCJKWF1IpniRI4A96gJiSn5rk9bA_d40bb5W92fo664TyrI1BCx627pfIIgWmWPC4'); //FCM_SERVER_KEY
    }
  }

  final FirebaseMessagingWrapper? _mc;
  StreamSubscription? _sub;

  @override
  Future requestPermissions() async {
    return await _mc?.requestPermission();
  }

  @override
  Future cancel() async {
    await _sub?.cancel();
  }

  Future close() async {
    super.closePushStreams();
    disposeAll();
  }

  @override
  Future initialize(MessageConverter messageConverter) async {
    _mc?.onTokenRefresh.auto(this, onItem: (_) async {
      final token = await FirebaseMessaging.instance.getToken();
      newPushToken(token);
    });
    super.convertMessage = messageConverter;
  }

  @override
  Future<String> internalGetToken() async => (await _mc?.getToken())!;

  @override
  Future subscribe(BuildContext context, {bool configureBackground = false}) async {
    _sub?.cancel();
    _sub = _mc?.onMessage.map((m) {
      log.info("Got a new web message!");
      return m;
    }).listen((msg) => newRawMessage(PushSource.foreground, msg), cancelOnError: false);
  }
}
