import 'package:flutter/cupertino.dart';
import 'package:flutter/foundation.dart';
import 'package:sunny_fluro/sunny_fluro.dart';
import 'package:sunny_lifecycle/modules/lifecycle_modules.dart';
import 'package:sunny_lifecycle/routing/route_history.dart';
import 'package:sunny_lifecycle/routing2/frouter_parser.dart';
import 'package:sunny_lifecycle/routing2/frouter_providers.dart';
import 'package:sunny_lifecycle/routing2/frouter_state.dart';
import 'package:sunny_lifecycle/tabs.dart';
import 'package:sunny_sdk_core/auth.dart';

import 'routing_system.dart';

import 'frouter_delegate.dart';

class FluroRoutingSystem implements RoutingSystem {
  bool _isDisposed = false;

  @override
  final BackButtonDispatcher backButtonDispatcher = RootBackButtonDispatcher();

  @override
  final FRouterDelegate delegate;

  @override
  final RouteInformationParser<Object> parser;

  @override
  final RouteInformationProvider? provider;

  final FRouteState state;

  FluroRoutingSystem._(
      {required this.delegate,
      required this.parser,
      required this.state,
      this.provider});

  @override
  void dispose() {
    if (!_isDisposed) {
      _isDisposed = true;
      delegate.dispose();
      state.dispose();
    }
  }
}

class FluroRoutingModule implements RoutingModule {
  static final globalNavigatorKey =
      GlobalKey<NavigatorState>(debugLabel: 'rootFluroSystem');

  final Stream<AuthUserProfile?> Function()? userChanges;
  final AuthUserProfile? Function()? currentUser;
  final TabSet? tabs;

  final String path;
  final FRouter routes;
  final String initialRoute;
  final GlobalKey<NavigatorState> navigatorKey;
  final bool preferUpdates;

  /// Observe history changes at the root router level
  final List<HistoryObserver> historyObservers;

  FluroRoutingModule({
    required this.path,
    GlobalKey<NavigatorState>? navigatorKey,
    this.preferUpdates = true,
    this.userChanges,
    this.currentUser,
    this.tabs,
    required this.routes,
    required this.initialRoute,
    this.historyObservers = const [],
  }) : navigatorKey = navigatorKey ?? globalNavigatorKey;

  @override
  Future initializeRoutes() {
    return SynchronousFuture(null);
  }

  @override
  RoutingSystem createRoutingSystem([RouterDelegate? parent]) {
    final state = FRouteState(historyObservers: historyObservers);
    var initialAppRoute = routes.matchRoute(initialRoute)!.route!;

    final delegate = (parent is FRouterDelegate)
        ? FRouterDelegate(state, routes,
            parent: parent,
            path: path,
            preferUpdates: preferUpdates,
            initialRoute: initialAppRoute,
            navigatorKey: navigatorKey,
            historyObservers: historyObservers)
        : FRouterDelegate.root(state, routes,
            historyObservers: historyObservers,
            preferUpdates: preferUpdates,
            path: path,
            initialRoute: initialAppRoute,
            navigatorKey: navigatorKey);

    return FluroRoutingSystem._(
      delegate: delegate,
      parser: FRouterInformationParser(routes, delegate),
      state: state,
      provider: FRouterRootProviderPlatform(
        state: state,
        initialRoute:
            RouteInformation(location: delegate.calculateInitialRoute()),
      ),
    );
  }
}
