import 'dart:async';

import 'package:equatable/equatable.dart';
import 'package:sunny_fluro/sunny_fluro.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:sunny_dart/sunny_dart.dart';
import 'package:sunny_lifecycle/routing2/frouter_delegate.dart';

AppRouteMatch rootAppRouteMatch(Widget splashPage()) => AppRouteMatch.ofMap(
      AppPageRoute('/', (context, _) => splashPage(), (_) => const DefaultRouteParams()),
      const {},
    );

class FRouterInformationParser with LoggingMixin implements RouteInformationParser<FRouteMatch> {
  /// Used to find other routers that might handle this route
  final FRouterDelegate delegate;
  final FRouter router;

  const FRouterInformationParser(this.router, this.delegate);

  String get loggerName => "routeParser";

  @override
  Future<FRouteMatch> parseRouteInformation(RouteInformation routeInformation) async {
    return delegate.matchRoute(routeInformation);
  }

  @override
  RouteInformation restoreRouteInformation(FRouteMatch configuration) {
    var routeUri = configuration.match.route?.routeUri(configuration.match.parameters);

    log.info("restore $routeUri");
    if (routeUri == null) {
      return RouteInformation(location: '/');
    } else {
      return RouteInformation(
        state: configuration.match.parameters,
        location: routeUri,
      );
    }
  }
}

class FRouteMatch extends Equatable {
  final AppRouteMatch match;
  final FRouterDelegate matchedBy;
  final bool isManaged;

  FRouteMatch.page(this.match, this.matchedBy) : isManaged = true;

  /// A route not tracked by the FRouter (pushed and popped separately)
  FRouteMatch.external(this.matchedBy)
      : match = AppRouteMatch.ofMap(ExternalAppRoute(), {}),
        isManaged = false;

  @override
  List<Object?> get props => [match, matchedBy.path, isManaged];
}

class ExternalAppRoute<R, P extends RouteParams> implements AppRoute<R, P> {
  final String name;

  ExternalAppRoute._(this.name);
  ExternalAppRoute() : this._("/${uuid}");

  @override
  String get route => name;

  @override
  String? routeTitle([RouteParams? params]) => "External route";

  @override
  String? routeUri(params) => name;

  @override
  ToRouteUri? get toRouteUri => null;

  @override
  ParameterConverter<P>? get paramConverter => null;

  @override
  AppRoute<RR, PP> cast<RR, PP extends RouteParams>() {
    return ExternalAppRoute<RR, PP>._(this.name);
  }
}
